/***************************************************************************
 *   Copyright (C) 2005 by Patrick Steiner                                 *
 *   patrick.steiner a1.net                                                *
 *   Copyright (C) Serdar Ozler (Scumm.QPE)                                *
 *   sozler@sitebest.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "mainwindow.h"
#include "newgamedlg.h"
#include "gamelist.h"

MainWindow::MainWindow()
           :QMainWindow()
{
  // construct objects
  tabs=new QTabWidget(this);
  setCentralWidget(tabs);
  tabStart=new QWidget(tabs);
  tabOptions=new QWidget(tabs);
  tabOutput=new QWidget(tabs);
  tabAbout=new QWidget(tabs);
  tabs->addTab(tabStart, tr("&Start"));
  tabs->addTab(tabOptions, tr("&Options"));
  tabs->addTab(tabOutput, tr("Out&put"));
  tabs->addTab(tabAbout, tr("&About"));

  listGames=new QListBox(tabStart);
  listGames->setGeometry(10, 40, 215, 175);
  connect(listGames, SIGNAL(highlighted(const QString &)), this, SLOT(highlighted_Games(const QString &)));
  label=new QLabel(listGames, tr("&Games:"), tabStart);
  label->setGeometry(10, 10, 215, 20);

  buttonStart=new QPushButton(tr("S&tart"), tabStart);
  buttonStart->setGeometry(10, 225, 100, 25);
  connect(buttonStart, SIGNAL(clicked()), this, SLOT(clicked_Start()));
  buttonExit=new QPushButton(tr("E&xit"), tabStart);
  buttonExit->setGeometry(125, 225, 100, 25);
  connect(buttonExit, SIGNAL(clicked()), this, SLOT(close()));

  spinMusic=new QSpinBox(0, 100, 1, tabOptions);
  spinMusic->setGeometry(10, 40, 50, 25);
  spinMusic->setValue(60);
  sliderMusic=new QSlider(0, 100, 10, 60, QSlider::Horizontal, tabOptions);
  sliderMusic->setGeometry(70, 40, 155, 25);
  sliderMusic->setTickmarks(QSlider::Below);
  connect(sliderMusic, SIGNAL(valueChanged(int)), spinMusic, SLOT(setValue(int)));
  label=new QLabel(spinMusic, tr("&Music Volume:"), tabOptions);
  label->setGeometry(10, 10, 215, 20);

  spinSFX=new QSpinBox(0, 255, 1, tabOptions);
  spinSFX->setGeometry(10, 105, 50, 25);
  spinSFX->setValue(100);
  sliderSFX=new QSlider(0, 255, 25, 100, QSlider::Horizontal, tabOptions);
  sliderSFX->setGeometry(70, 105, 155, 25);
  sliderSFX->setTickmarks(QSlider::Below);
  connect(sliderSFX, SIGNAL(valueChanged(int)), spinSFX, SLOT(setValue(int)));
  label=new QLabel(spinSFX, tr("SFX &Volume:"), tabOptions);
  label->setGeometry(10, 75, 215, 20);

  spinText=new QSpinBox(5, 90, 1, tabOptions);
  spinText->setGeometry(10, 170, 50, 25);
  spinText->setValue(60);
  sliderText=new QSlider(5, 90, 9, 60, QSlider::Horizontal, tabOptions);
  sliderText->setGeometry(70, 170, 155, 25);
  sliderText->setTickmarks(QSlider::Below);
  connect(sliderText, SIGNAL(valueChanged(int)), spinText, SLOT(setValue(int)));
  label=new QLabel(spinMusic, tr("&Talk Volume:"), tabOptions);
  label->setGeometry(10, 140, 215, 20);

  checkSubtitles=new QCheckBox("&Enable Subtitles", tabOptions);
  checkSubtitles->setGeometry(10, 210, 215, 25);
  checkSubtitles->setChecked(TRUE);

  layout=new QVBoxLayout(tabOutput);
  editOutput=new QMultiLineEdit(tabOutput);
  layout->addWidget(editOutput);

  static const QString programVersion = "1.0.1";
  static const QString programName = "Opie-Scummvm";

  static const QString aboutText = programName + "\n" + 
                                   "Graphical User Interface for ScummVM" + "\n" +
                                   "Developed by: Patrick Steiner" + "\n" +
                                   "Developed by: Serdar Ozler (Scumm.QPE)" + "\n\n" +
                                   "Release: " + programVersion + "\n" +
                                   "Release Date: April 15, 2005\n\n" +
                                   "INFOS:\n" +
                                   "MENU-BUTTON: Get Scummvm-Mainmenu\n" +
                                   "Cancel-BUTTON: Skip movies\n" +
                                   "SPACE: Pause game";

  layout=new QVBoxLayout(tabAbout);
  label=new QLabel(tr(aboutText), tabAbout);
  label->setGeometry(tabAbout->rect());
  label->setAlignment(Qt::AlignCenter | Qt::WordBreak);
  layout->addWidget(label);

  // set window title
  setCaption(tr(programName));

  // read initial configuration
  configuration.setAutoDelete(TRUE);
  readConfiguration();
}

MainWindow::~MainWindow()
{
}

void MainWindow::readConfiguration()
{
  configuration.clear();
  QFile file(getenv("HOME")+QString("/.scummvmrc"));
  if (file.open(IO_ReadOnly))
  {
    QTextStream stream(&file);
    QString domain;
    while (!stream.atEnd())
    {
      QString line=stream.readLine().stripWhiteSpace();
      if (!line.isNull() && !line.isEmpty())
      {
        int pos;
        if (line[0]=='[')
          domain=line.mid(1, line.find(']', 1)-1);
        else
          if ((pos=line.find('='))>0 && !domain.isNull())
          {
            configOption *option=new configOption;
            option->domain=domain;
            option->key=line.left(pos);
            option->value=line.mid(pos+1);
            configuration.append(option);
          }
      }
    }
    file.close();
  }
  constructListGames();
}

void MainWindow::constructListGames()
{
  listGames->clear();
  for (configOption *option=configuration.first(); option; option=configuration.next())
    if (option->key=="path" && !listGames->findItem(detectGameName(option->domain)))
      listGames->insertItem(detectGameName(option->domain));
  listGames->insertItem("Not in list...");
  if (listGames->currentItem()>=0)
    highlighted_Games(listGames->currentText());
}

QString MainWindow::detectGameName(QString fileName)
{
  const VersionSettings *game=version_settings;
  do
    if (fileName==game->filename) return game->gamename;
  while ((++game)->filename);
  return "Unknown Game";
}

QString MainWindow::detectFileName(QString gameName)
{
  const VersionSettings *game=version_settings;
  do
    if (gameName==game->gamename) return game->filename;
  while ((++game)->filename);
  return "";
}

QString MainWindow::getOptionValue(QString domain, QString key, QString defaultValue)
{
  for (configOption *option=configuration.first(); option; option=configuration.next())
    if (option->domain==domain && option->key==key)
      return option->value;
  return defaultValue;
}

void MainWindow::highlighted_Games(const QString &gameName)
{
  QString domain=detectFileName(gameName);
  sliderMusic->setValue(getOptionValue(domain, "music_volume", "60").toInt());
  sliderSFX->setValue(getOptionValue(domain, "sfx_volume", "100").toInt());
  sliderText->setValue(getOptionValue(domain, "talk_volume", "60").toInt());
  checkSubtitles->setChecked(getOptionValue(domain, "nosubtitles", "false")=="false");
}

void MainWindow::clicked_Start()
{
  QString domain(detectFileName(listGames->currentText())), path;
  if (domain.isEmpty())
  {
    NewGameDialog dlgNewGame(this);
    if (dlgNewGame.exec()==QDialog::Accepted)
    {
      domain=detectFileName(dlgNewGame.comboGames->currentText());
      path=dlgNewGame.editDirectory->text();
    }
    else
      return;
  }
  else
    path=getOptionValue(domain, "path", "/opt/QtPalmtop/share/games/scummvm/"+domain+"/");

  editOutput->clear();
  tabs->showPage(tabOutput);
  
  // new command string 
  //QString cmd("scummvm -w -gnormal -p"+path+" -m"+QString::number(spinMusic->value())+" -s"+QString::number(spinSFX->value())+" -y"+QString::number(spinText->value()));
  QString cmd("scummvm -gnormal -p"+path+" -m"+QString::number(spinMusic->value())+" -s"+QString::number(spinSFX->value())+" --talkspeed="+QString::number(spinText->value()));
  if (!checkSubtitles->isChecked()) cmd+=" -n";
  cmd+=" "+domain+" 2>&1";
  editOutput->insertLine("Executing \""+cmd+"\"");

  FILE *fp=popen(cmd.latin1(), "r");
  if (!fp)
    editOutput->insertLine("An unknown error occured!");
  else
  {
    QTextIStream stream(fp);
    while (!stream.atEnd())
      editOutput->insertLine(stream.readLine());
  }
  pclose(fp);

  readConfiguration();
}
